@@ -28,6 +28,7 @@
 #include "progressindicator.h"
 #include "treemodel.h"
 
+#include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 
 #include <QDebug>
@@ -39,6 +40,7 @@
 #include <QMenu>
 #include <QMouseEvent>
 #include <QSettings>
+#include <QSortFilterProxyModel>
 #include <QTimer>
 
 namespace Utils {
@@ -50,15 +52,53 @@ class BaseTreeViewPrivate : public QObject
 {
 public:
     explicit BaseTreeViewPrivate(BaseTreeView *parent)
-        : q(parent), m_settings(0), m_expectUserChanges(false), m_progressIndicator(0)
-    {}
+        : q(parent)
+    {
+        m_settingsTimer.setSingleShot(true);
+        connect(&m_settingsTimer, &QTimer::timeout,
+                this, &BaseTreeViewPrivate::doSaveState);
+        connect(q->header(), &QHeaderView::sectionResized, this, [this](int logicalIndex, int oldSize, int newSize) {
+            if (m_processingSpans || m_spanColumn < 0)
+                return;
 
-    bool eventFilter(QObject *, QEvent *event)
+            QHeaderView *h = q->header();
+            QTC_ASSERT(h, return);
+
+            // Last non-hidden column.
+            int count = h->count();
+            while (count > 0 && h->isSectionHidden(count - 1))
+                --count;
+
+            if (count == 0)
+                return;
+
+            int column = logicalIndex;
+            if (oldSize < newSize)
+            {
+                // Protect against sizing past the next section.
+                while (column + 1 < count && h->sectionSize(column + 1) == h->minimumSectionSize())
+                    ++column;
+            }
+
+            if (logicalIndex >= m_spanColumn) {
+                // Resize after the span column.
+                column = column + 1;
+            } else {
+                // Resize the span column or before it.
+                column = m_spanColumn;
+            }
+
+            rebalanceColumns(column, false);
+        });
+        connect(q->header(), &QHeaderView::geometriesChanged, this, QOverload<>::of(&BaseTreeViewPrivate::rebalanceColumns));
+    }
+
+    bool eventFilter(QObject *, QEvent *event) override
     {
         if (event->type() == QEvent::MouseMove) {
             // At this time we don't know which section will get which size.
             // But we know that a resizedSection() will be emitted later.
-            QMouseEvent *me = static_cast<QMouseEvent *>(event);
+            const auto *me = static_cast<QMouseEvent *>(event);
             if (me->buttons() & Qt::LeftButton)
                 m_expectUserChanges = true;
         }
@@ -102,6 +142,12 @@ public:
 
     void saveState()
     {
+        m_settingsTimer.start(2000); // Once per 2 secs should be enough.
+    }
+
+    void doSaveState()
+    {
+        m_settingsTimer.stop();
         if (m_settings && !m_settingsKey.isEmpty()) {
             m_settings->beginGroup(m_settingsKey);
             QVariantList l;
@@ -137,7 +183,7 @@ public:
         QAbstractItemModel *m = q->model();
         for (int i = 0; i < 100 && a.isValid(); ++i) {
             const QString s = m->data(a).toString();
-            int w = fm.width(s) + 10;
+            int w = fm.horizontalAdvance(s) + 10;
             if (column == 0) {
                 for (QModelIndex b = a.parent(); b.isValid(); b = b.parent())
                     w += ind;
@@ -158,7 +204,8 @@ public:
         QTC_ASSERT(m, return -1);
 
         QFontMetrics fm = q->fontMetrics();
-        int minimum = fm.width(m->headerData(column, Qt::Horizontal).toString()) + 2 * fm.width(QLatin1Char('m'));
+        int minimum = fm.horizontalAdvance(m->headerData(column, Qt::Horizontal).toString())
+            + 2 * fm.horizontalAdvance(QLatin1Char('m'));
         considerItems(column, q->indexAt(QPoint(1, 1)), &minimum, false);
 
         QVariant extraIndices = m->data(QModelIndex(), BaseTreeView::ExtraIndicesForColumnWidth);
@@ -187,6 +234,17 @@ public:
         }
     }
 
+    void setSpanColumn(int column)
+    {
+        if (column == m_spanColumn)
+            return;
+
+        m_spanColumn = column;
+        if (m_spanColumn >= 0)
+            q->header()->setStretchLastSection(false);
+        rebalanceColumns();
+    }
+
     void toggleColumnWidth(int logicalIndex)
     {
         QHeaderView *h = q->header();
@@ -197,22 +255,86 @@ public:
         // when we have that size already, in that case minimize.
         if (currentSize == suggestedSize) {
             QFontMetrics fm = q->fontMetrics();
-            int headerSize = fm.width(q->model()->headerData(logicalIndex, Qt::Horizontal).toString());
-            int minSize = 10 * fm.width(QLatin1Char('x'));
+            int headerSize = fm.horizontalAdvance(q->model()->headerData(logicalIndex, Qt::Horizontal).toString());
+            int minSize = 10 * fm.horizontalAdvance(QLatin1Char('x'));
             targetSize = qMax(minSize, headerSize);
         }
+
+        // Prevent rebalance as part of this resize.
+        m_processingSpans = true;
         h->resizeSection(logicalIndex, targetSize);
+        m_processingSpans = false;
+
+        // Now trigger a rebalance so it resizes the span column. (if set)
+        rebalanceColumns();
+
         m_userHandled.remove(logicalIndex); // Reset.
         saveState();
     }
 
+    void rebalanceColumns()
+    {
+        rebalanceColumns(m_spanColumn, true);
+    }
+
+    void rebalanceColumns(int column, bool allowResizePrevious)
+    {
+        if (m_spanColumn < 0 || column < 0 || m_processingSpans)
+            return;
+
+        QHeaderView *h = q->header();
+        QTC_ASSERT(h, return);
+
+        int count = h->count();
+        if (column >= count)
+            return;
+
+        // Start with the target column, and resize other columns as necessary.
+        int totalSize = q->viewport()->width();
+        if (tryRebalanceColumns(column, totalSize))
+            return;
+
+        for (int i = allowResizePrevious ? 0 : column + 1; i < count; ++i) {
+            if (i != column && tryRebalanceColumns(i, totalSize))
+                return;
+        }
+    }
+
+    bool tryRebalanceColumns(int column, int totalSize)
+    {
+        QHeaderView *h = q->header();
+
+        int count = h->count();
+        int otherColumnTotal = 0;
+        for (int i = 0; i < count; ++i) {
+            if (i != column)
+                otherColumnTotal += h->sectionSize(i);
+        }
+
+        if (otherColumnTotal < totalSize) {
+            m_processingSpans = true;
+            h->resizeSection(column, totalSize - otherColumnTotal);
+            m_processingSpans = false;
+        } else
+            return false;
+
+        // Make sure this didn't go over the total size.
+        int totalColumnSize = 0;
+        for (int i = 0; i < count; ++i)
+            totalColumnSize += h->sectionSize(i);
+        return totalColumnSize == totalSize;
+    }
+
 public:
     BaseTreeView *q;
     QMap<int, int> m_userHandled; // column -> width, "not present" means "automatic"
-    QSettings *m_settings;
+    QSettings *m_settings = nullptr;
+    QTimer m_settingsTimer;
     QString m_settingsKey;
-    bool m_expectUserChanges;
-    ProgressIndicator *m_progressIndicator;
+    bool m_expectUserChanges = false;
+    ProgressIndicator *m_progressIndicator = nullptr;
+    int m_spanColumn = -1;
+    bool m_processingSpans = false;
 };
 
 class BaseTreeViewDelegate : public QItemDelegate
@@ -221,7 +343,7 @@ public:
     BaseTreeViewDelegate(QObject *parent): QItemDelegate(parent) {}
 
     QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option,
-                          const QModelIndex &index) const
+                          const QModelIndex &index) const override
     {
         Q_UNUSED(option);
         QLabel *label = new QLabel(parent);
@@ -272,14 +394,18 @@ BaseTreeView::~BaseTreeView()
 
 void BaseTreeView::setModel(QAbstractItemModel *m)
 {
-    if (BaseTreeModel *oldModel = qobject_cast<BaseTreeModel *>(model()))
+    if (auto oldModel = qobject_cast<BaseTreeModel *>(model())) {
         disconnect(oldModel, &BaseTreeModel::requestExpansion, this, &BaseTreeView::expand);
+        disconnect(oldModel, &BaseTreeModel::requestCollapse, this, &BaseTreeView::collapse);
+    }
 
     TreeView::setModel(m);
 
     if (m) {
-        if (BaseTreeModel *newModel = qobject_cast<BaseTreeModel *>(m))
+        if (auto newModel = qobject_cast<BaseTreeModel *>(m)) {
             connect(newModel, &BaseTreeModel::requestExpansion, this, &BaseTreeView::expand);
+            connect(newModel, &BaseTreeModel::requestCollapse, this, &BaseTreeView::collapse);
+        }
         d->restoreState();
 
         QVariant delegateBlob = m->data(QModelIndex(), ItemDelegateRole);
@@ -293,10 +419,31 @@ void BaseTreeView::setModel(QAbstractItemModel *m)
 
 void BaseTreeView::mousePressEvent(QMouseEvent *ev)
 {
-    TreeView::mousePressEvent(ev);
-    const QModelIndex mi = indexAt(ev->pos());
-    if (!mi.isValid())
-        d->toggleColumnWidth(columnAt(ev->x()));
+    ItemViewEvent ive(ev, this);
+    QTC_ASSERT(model(), return);
+    if (!model()->setData(ive.index(), QVariant::fromValue(ive), ItemViewEventRole))
+        TreeView::mousePressEvent(ev);
+// Resizing columns by clicking on the empty space seems to be controversial.
+// Let's try without for a while.
+//    const QModelIndex mi = indexAt(ev->pos());
+//    if (!mi.isValid())
+//        d->toggleColumnWidth(columnAt(ev->x()));
+}
+
+void BaseTreeView::mouseMoveEvent(QMouseEvent *ev)
+{
+    ItemViewEvent ive(ev, this);
+    QTC_ASSERT(model(), return);
+    if (!model()->setData(ive.index(), QVariant::fromValue(ive), ItemViewEventRole))
+        TreeView::mouseMoveEvent(ev);
+}
+
+void BaseTreeView::mouseReleaseEvent(QMouseEvent *ev)
+{
+    ItemViewEvent ive(ev, this);
+    QTC_ASSERT(model(), return);
+    if (!model()->setData(ive.index(), QVariant::fromValue(ive), ItemViewEventRole))
+        TreeView::mouseReleaseEvent(ev);
 }
 
 void BaseTreeView::contextMenuEvent(QContextMenuEvent *ev)
@@ -341,6 +488,12 @@ void BaseTreeView::mouseDoubleClickEvent(QMouseEvent *ev)
         TreeView::mouseDoubleClickEvent(ev);
 }
 
+void BaseTreeView::resizeEvent(QResizeEvent *ev)
+{
+    TreeView::resizeEvent(ev);
+    d->rebalanceColumns();
+}
+
 void BaseTreeView::showEvent(QShowEvent *ev)
 {
     emit aboutToShow();
@@ -355,7 +508,7 @@ void BaseTreeView::showEvent(QShowEvent *ev)
 void BaseTreeView::showProgressIndicator()
 {
     if (!d->m_progressIndicator) {
-        d->m_progressIndicator = new ProgressIndicator(ProgressIndicator::Large);
+        d->m_progressIndicator = new ProgressIndicator(ProgressIndicatorSize::Large);
         d->m_progressIndicator->attachToWidget(this);
     }
     d->m_progressIndicator->show();
@@ -388,6 +541,21 @@ void BaseTreeView::resizeColumns()
     d->resizeColumns();
 }
 
+int BaseTreeView::spanColumn() const
+{
+    return d->m_spanColumn;
+}
+
+void BaseTreeView::setSpanColumn(int column)
+{
+    d->setSpanColumn(column);
+}
+
+void BaseTreeView::refreshSpanColumn()
+{
+    d->rebalanceColumns();
+}
+
 void BaseTreeView::setSettings(QSettings *settings, const QByteArray &key)
 {
     QTC_ASSERT(!d->m_settings, qDebug() << "DUPLICATED setSettings" << key);
@@ -418,21 +586,35 @@ ItemViewEvent::ItemViewEvent(QEvent *ev, QAbstractItemView *view)
         m_index = view->indexAt(m_pos);
         break;
     default:
-        m_index = selection->currentIndex();
+        m_index = selection ? selection->currentIndex() : QModelIndex();
         break;
     }
 
-    m_selectedRows = selection->selectedRows();
-    if (m_selectedRows.isEmpty()) {
-        QModelIndex current = selection->currentIndex();
-        if (current.isValid())
-            m_selectedRows.append(current);
+    if (selection) {
+        m_selectedRows = selection->selectedRows();
+        if (m_selectedRows.isEmpty()) {
+            QModelIndex current = selection->currentIndex();
+            if (current.isValid())
+                m_selectedRows.append(current);
+        }
     }
+
+    auto fixIndex = [view](QModelIndex idx) {
+        QAbstractItemModel *model = view->model();
+        while (auto proxy = qobject_cast<QSortFilterProxyModel *>(model)) {
+            idx = proxy->mapToSource(idx);
+            model = proxy->sourceModel();
+        }
+        return idx;
+    };
+
+    m_sourceModelIndex = fixIndex(m_index);
+    m_selectedRows = Utils::transform(m_selectedRows, fixIndex);
 }
 
 QModelIndexList ItemViewEvent::currentOrSelectedRows() const
 {
-    return m_selectedRows.isEmpty() ? QModelIndexList() << m_index : m_selectedRows;
+    return m_selectedRows.isEmpty() ? QModelIndexList() << m_sourceModelIndex : m_selectedRows;
 }
 
 } // namespace Utils
