@@ -33,6 +33,7 @@
 #include <projectexplorer/devicesupport/deviceprocess.h>
 #include <projectexplorer/kit.h>
 #include <projectexplorer/kitinformation.h>
+#include <projectexplorer/runcontrol.h>
 #include <projectexplorer/target.h>
 
 #include <utils/qtcprocess.h>
@@ -75,15 +76,14 @@ PerfConfigWidget::PerfConfigWidget(PerfSettings *settings, QWidget *parent) :
     m_ui->sampleMode->addItem(tr("frequency (Hz)"), QLatin1String(Constants::PerfSampleFrequency));
     m_ui->sampleMode->addItem(tr("event count"), QLatin1String(Constants::PerfSampleCount));
 
-    auto comboboxChangedSignal
-            = static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged);
+    auto comboboxChangedSignal = QOverload<int>::of(&QComboBox::currentIndexChanged);
     connect(m_ui->callgraphMode, comboboxChangedSignal, this, [this](int index) {
         QString mode = m_ui->callgraphMode->itemData(index).toString();
         m_settings->setCallgraphMode(mode);
         m_ui->stackSize->setEnabled(mode == QLatin1String(Constants::PerfCallgraphDwarf));
     });
 
-    auto spinBoxChangedSignal = static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged);
+    auto spinBoxChangedSignal = QOverload<int>::of(&QSpinBox::valueChanged);
     connect(m_ui->stackSize, spinBoxChangedSignal, m_settings, &PerfSettings::setStackSize);
     connect(m_ui->period, spinBoxChangedSignal, m_settings, &PerfSettings::setPeriod);
     connect(m_ui->sampleMode, comboboxChangedSignal, this, [this](int index) {
@@ -147,7 +147,7 @@ void PerfConfigWidget::setTarget(ProjectExplorer::Target *target)
     ProjectExplorer::IDevice::ConstPtr device;
     if (target) {
         if (ProjectExplorer::Kit *kit = target->kit())
-            device = ProjectExplorer::DeviceKitInformation::device(kit);
+            device = ProjectExplorer::DeviceKitAspect::device(kit);
     }
 
     if (device.isNull()) {
