@@ -32,6 +32,7 @@
 #include <coreplugin/icore.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/helpitem.h>
 #include <coreplugin/helpmanager.h>
 #include <utils/qtcassert.h>
 #include <extensionsystem/pluginmanager.h>
@@ -42,10 +43,9 @@
 #include <qmljs/parser/qmljsast_p.h>
 #include <qmljs/parser/qmljsastfwd_p.h>
 #include <qmljs/qmljsutils.h>
-#include <qmljs/qmljsqrcparser.h>
 #include <texteditor/texteditor.h>
-#include <texteditor/helpitem.h>
 #include <utils/executeondestruction.h>
+#include <utils/qrcparser.h>
 #include <utils/tooltip/tooltip.h>
 
 #include <QDir>
@@ -72,11 +72,10 @@ namespace {
 
     AST::UiObjectInitializer *nodeInitializer(AST::Node *node)
     {
-        AST::UiObjectInitializer *initializer = 0;
-        if (const AST::UiObjectBinding *binding = AST::cast<const AST::UiObjectBinding *>(node))
+        AST::UiObjectInitializer *initializer = nullptr;
+        if (auto binding = AST::cast<const AST::UiObjectBinding*>(node))
             initializer = binding->initializer;
-         else if (const AST::UiObjectDefinition *definition =
-                  AST::cast<const AST::UiObjectDefinition *>(node))
+         else if (auto definition = AST::cast<const AST::UiObjectDefinition*>(node))
             initializer = definition->initializer;
         return initializer;
     }
@@ -92,7 +91,7 @@ namespace {
     }
 }
 
-QmlJSHoverHandler::QmlJSHoverHandler() : m_modelManager(0)
+QmlJSHoverHandler::QmlJSHoverHandler()
 {
     m_modelManager = ModelManagerInterface::instance();
 }
@@ -134,7 +133,7 @@ static inline QString getModuleName(const ScopeChain &scopeChain, const Document
             const QString name = relativeDir.replace(QLatin1Char('/'), QLatin1Char('.'));
             return name;
         } else if (importInfo.isValid() && importInfo.type() == ImportType::QrcDirectory) {
-            QString path = QrcParser::normalizedQrcDirectoryPath(importInfo.path());
+            QString path = Utils::QrcParser::normalizedQrcDirectoryPath(importInfo.path());
             path = path.mid(1, path.size() - ((path.size() > 1) ? 2 : 1));
             const QString name = path.replace(QLatin1Char('/'), QLatin1Char('.'));
             return name;
@@ -148,36 +147,27 @@ bool QmlJSHoverHandler::setQmlTypeHelp(const ScopeChain &scopeChain, const Docum
 {
     QString moduleName = getModuleName(scopeChain, qmlDocument, value);
 
-    QMap<QString, QUrl> urlMap;
+    QStringList helpIdCandidates;
 
-    QString helpId;
-    do {
-        QStringList helpIdPieces(qName);
-        helpIdPieces.prepend(moduleName);
-        helpIdPieces.prepend(QLatin1String("QML"));
-        helpId = helpIdPieces.join(QLatin1Char('.'));
-        urlMap = HelpManager::linksForIdentifier(helpId);
-        if (!urlMap.isEmpty())
-            break;
-        if (helpIdPieces.size() > 3) {
-            QString lm = helpIdPieces.value(2);
-            helpIdPieces.removeAt(2);
-            helpId = helpIdPieces.join(QLatin1Char('.'));
-            urlMap = HelpManager::linksForIdentifier(helpId);
-            if (!urlMap.isEmpty())
-                break;
-            helpIdPieces.replace(1, lm);
-            urlMap = HelpManager::linksForIdentifier(helpId);
-            if (!urlMap.isEmpty())
-                break;
-        }
-        helpIdPieces.removeAt(1);
-        helpId = helpIdPieces.join(QLatin1Char('.'));
-        urlMap = HelpManager::linksForIdentifier(helpId);
-        if (!urlMap.isEmpty())
-            break;
-        return false;
-    } while (0);
+    QStringList helpIdPieces(qName);
+    helpIdPieces.prepend(moduleName);
+    helpIdPieces.prepend("QML");
+    helpIdCandidates += helpIdPieces.join('.');
+
+    if (helpIdPieces.size() > 3) {
+        QString lm = helpIdPieces.value(2);
+        helpIdPieces.removeAt(2);
+        helpIdCandidates += helpIdPieces.join('.');
+
+        helpIdPieces.replace(1, lm);
+        helpIdCandidates += helpIdPieces.join('.');
+    }
+
+    helpIdPieces.removeAt(1);
+    helpIdCandidates += helpIdPieces.join('.');
+
+    const HelpItem helpItem(helpIdCandidates, qName.join('.'), HelpItem::QmlComponent);
+    const HelpItem::Links links = helpItem.links();
 
     // Check if the module name contains a major version.
     QRegularExpression version("^([^\\d]*)(\\d+)\\.*\\d*$");
@@ -185,20 +175,21 @@ bool QmlJSHoverHandler::setQmlTypeHelp(const ScopeChain &scopeChain, const Docum
     if (m.hasMatch()) {
         QMap<QString, QUrl> filteredUrlMap;
         QStringRef maj = m.capturedRef(2);
-        for (auto x = urlMap.begin(); x != urlMap.end(); ++x) {
-            QString urlModuleName = x.value().path().split('/')[1];
+        for (const HelpItem::Link &link : links) {
+            QString urlModuleName = link.second.path().split('/')[1];
             if (urlModuleName.contains(maj))
-                filteredUrlMap.insert(x.key(), x.value());
+                filteredUrlMap.insert(link.first, link.second);
         }
         if (!filteredUrlMap.isEmpty()) {
-            // Use the url as helpId, to disambiguate different versions
-            helpId = filteredUrlMap.first().toString();
-            const HelpItem helpItem(helpId, qName.join(QLatin1Char('.')), HelpItem::QmlComponent, filteredUrlMap);
+            // Use the URL, to disambiguate different versions
+            const HelpItem helpItem(filteredUrlMap.first(),
+                                    qName.join(QLatin1Char('.')),
+                                    HelpItem::QmlComponent);
             setLastHelpItemIdentified(helpItem);
             return true;
         }
     }
-    setLastHelpItemIdentified(HelpItem(helpId, qName.join(QLatin1Char('.')), HelpItem::QmlComponent));
+    setLastHelpItemIdentified(helpItem);
     return true;
 }
 
@@ -211,7 +202,7 @@ void QmlJSHoverHandler::identifyMatch(TextEditorWidget *editorWidget, int pos, R
     if (!m_modelManager)
         return;
 
-    QmlJSEditorWidget *qmlEditor = qobject_cast<QmlJSEditorWidget *>(editorWidget);
+    auto qmlEditor = qobject_cast<QmlJSEditorWidget*>(editorWidget);
     QTC_ASSERT(qmlEditor, return);
 
     const QmlJSTools::SemanticInfo &semanticInfo = qmlEditor->qmlJsEditorDocument()->semanticInfo();
@@ -230,7 +221,7 @@ void QmlJSHoverHandler::identifyMatch(TextEditorWidget *editorWidget, int pos, R
     if (rangePath.isEmpty()) {
         // Is the cursor on an import? The ast path will have an UiImport
         // member in the last or second to last position!
-        AST::UiImport *import = 0;
+        AST::UiImport *import = nullptr;
         if (astPath.size() >= 1)
             import = AST::cast<AST::UiImport *>(astPath.last());
         if (!import && astPath.size() >= 2)
@@ -305,7 +296,7 @@ bool QmlJSHoverHandler::matchColorItem(const ScopeChain &scopeChain,
     if (!initializer)
         return false;
 
-    AST::UiObjectMember *member = 0;
+    AST::UiObjectMember *member = nullptr;
     for (AST::UiObjectMemberList *list = initializer->members; list; list = list->next) {
         if (posIsInSource(pos, list->member)) {
             member = list->member;
@@ -316,8 +307,8 @@ bool QmlJSHoverHandler::matchColorItem(const ScopeChain &scopeChain,
         return false;
 
     QString color;
-    const Value *value = 0;
-    if (const AST::UiScriptBinding *binding = AST::cast<const AST::UiScriptBinding *>(member)) {
+    const Value *value = nullptr;
+    if (auto binding = AST::cast<const AST::UiScriptBinding *>(member)) {
         if (binding->qualifiedId && posIsInSource(pos, binding->statement)) {
             value = scopeChain.evaluate(binding->qualifiedId);
             if (value && value->asColorValue()) {
@@ -326,8 +317,7 @@ bool QmlJSHoverHandler::matchColorItem(const ScopeChain &scopeChain,
                                binding->statement->lastSourceLocation());
             }
         }
-    } else if (const AST::UiPublicMember *publicMember =
-               AST::cast<const AST::UiPublicMember *>(member)) {
+    } else if (auto publicMember = AST::cast<const AST::UiPublicMember *>(member)) {
         if (!publicMember->name.isEmpty() && posIsInSource(pos, publicMember->statement)) {
             value = scopeChain.lookup(publicMember->name.toString());
             if (const Reference *ref = value->asReference())
@@ -440,33 +430,33 @@ void QmlJSHoverHandler::prettyPrintTooltip(const Value *value,
 static const ObjectValue *isMember(const ScopeChain &scopeChain,
                                                 AST::Node *node, QString *name)
 {
-    const ObjectValue *owningObject = 0;
-    if (AST::IdentifierExpression *identExp = AST::cast<AST::IdentifierExpression *>(node)) {
+    const ObjectValue *owningObject = nullptr;
+    if (auto identExp = AST::cast<const AST::IdentifierExpression *>(node)) {
         if (identExp->name.isEmpty())
-            return 0;
+            return nullptr;
         *name = identExp->name.toString();
         scopeChain.lookup(*name, &owningObject);
-    } else if (AST::FieldMemberExpression *fme = AST::cast<AST::FieldMemberExpression *>(node)) {
+    } else if (auto fme = AST::cast<const AST::FieldMemberExpression *>(node)) {
         if (!fme->base || fme->name.isEmpty())
-            return 0;
+            return nullptr;
         *name = fme->name.toString();
         const Value *base = scopeChain.evaluate(fme->base);
         if (!base)
-            return 0;
+            return nullptr;
         owningObject = base->asObjectValue();
         if (owningObject)
             owningObject->lookupMember(*name, scopeChain.context(), &owningObject);
-    } else if (AST::UiQualifiedId *qid = AST::cast<AST::UiQualifiedId *>(node)) {
+    } else if (auto qid = AST::cast<const AST::UiQualifiedId *>(node)) {
         if (qid->name.isEmpty())
-            return 0;
+            return nullptr;
         *name = qid->name.toString();
         const Value *value = scopeChain.lookup(*name, &owningObject);
         for (AST::UiQualifiedId *it = qid->next; it; it = it->next) {
             if (!value)
-                return 0;
+                return nullptr;
             const ObjectValue *next = value->asObjectValue();
             if (!next || it->name.isEmpty())
-                return 0;
+                return nullptr;
             *name = it->name.toString();
             value = next->lookupMember(*name, scopeChain.context(), &owningObject);
         }
@@ -483,7 +473,7 @@ bool QmlJSHoverHandler::setQmlHelpItem(const ScopeChain &scopeChain,
     if (const ObjectValue *scope = isMember(scopeChain, node, &name)) {
         // maybe it's a type?
         if (!name.isEmpty() && name.at(0).isUpper()) {
-            if (AST::UiQualifiedId *qualifiedId = AST::cast<AST::UiQualifiedId *>(node)) {
+            if (auto qualifiedId = AST::cast<AST::UiQualifiedId *>(node)) {
                 const ObjectValue *value = scopeChain.context()->lookupType(qmlDocument.data(), qualifiedId);
                 if (setQmlTypeHelp(scopeChain, qmlDocument, value, QStringList(qualifiedId->name.toString())))
                     return true;
@@ -500,22 +490,13 @@ bool QmlJSHoverHandler::setQmlHelpItem(const ScopeChain &scopeChain,
             const QString className = cur->className();
             if (!className.isEmpty()) {
                 moduleName = getModuleName(scopeChain, qmlDocument, cur);
-                QString helpId;
-                do {
-                    helpId = QLatin1String("QML.") + moduleName + QLatin1Char('.') + className
-                            + QLatin1String("::") + name;
-                    if (!HelpManager::linksForIdentifier(helpId).isEmpty())
-                        break;
-                    helpId = QLatin1String("QML.") + className + QLatin1String("::") + name;
-                    if (!HelpManager::linksForIdentifier(helpId).isEmpty())
-                        break;
-                    helpId = className + QLatin1String("::") + name;
-                    if (!HelpManager::linksForIdentifier(helpId).isEmpty())
-                        break;
-                    helpId.clear();
-                } while (0);
-                if (!helpId.isEmpty()) {
-                    setLastHelpItemIdentified(HelpItem(helpId, name, HelpItem::QmlProperty));
+                const QStringList helpIdCandidates = {"QML." + moduleName + '.' + className
+                                                          + "::" + name,
+                                                      "QML." + className + "::" + name,
+                                                      className + "::" + name};
+                const HelpItem helpItem(helpIdCandidates, name, HelpItem::QmlProperty);
+                if (helpItem.isValid()) {
+                    setLastHelpItemIdentified(helpItem);
                     return true;
                 }
             }
@@ -525,7 +506,7 @@ bool QmlJSHoverHandler::setQmlHelpItem(const ScopeChain &scopeChain,
         }
     } else {
         // it might be a type, but the scope chain is broken (mismatched braces)
-        if (AST::UiQualifiedId *qualifiedId = AST::cast<AST::UiQualifiedId *>(node)) {
+        if (auto qualifiedId = AST::cast<AST::UiQualifiedId *>(node)) {
             const ObjectValue *value = scopeChain.context()->lookupType(qmlDocument.data(),
                                                                         qualifiedId);
             if (setQmlTypeHelp(scopeChain, qmlDocument, value,
