@@ -35,38 +35,32 @@
 #include <builddependency.h>
 #include <clangcodemodelclientmessages.h>
 #include <clangcodemodelservermessages.h>
-#include <clangdocumentsuspenderresumer.h>
 #include <clangpathwatcher.h>
 #include <clangrefactoringmessages.h>
-#include <clangreferencescollector.h>
-#include <filestatus.h>
 #include <filepath.h>
 #include <filepathcaching.h>
-#include <fulltokeninfo.h>
+#include <filepathview.h>
+#include <filestatus.h>
 #include <includesearchpath.h>
 #include <nativefilepath.h>
+#include <pchpaths.h>
 #include <pchtask.h>
 #include <precompiledheadersupdatedmessage.h>
 #include <projectpartartefact.h>
-#include <projectpartid.h>
+#include <projectpartentry.h>
+#include <projectpartpch.h>
 #include <sourcedependency.h>
 #include <sourcelocationentry.h>
 #include <sourcelocationscontainer.h>
-#include <tokenprocessor.h>
-#include <filepathview.h>
+#include <symbol.h>
 #include <symbolentry.h>
 #include <symbolindexertaskqueue.h>
-#include <symbol.h>
-#include <tooltipinfo.h>
 #include <toolchainargumentscache.h>
-#include <projectpartentry.h>
+#include <tooltipinfo.h>
 #include <usedmacro.h>
-
 #include <cpptools/usages.h>
-
 #include <projectexplorer/projectmacro.h>
 #include <projectexplorer/headerpath.h>
-
 #include <coreplugin/find/searchresultitem.h>
 #include <coreplugin/locator/ilocatorfilter.h>
 
@@ -193,6 +187,8 @@ const char * toText(Utils::Language language)
         return "C";
     case Language::Cxx:
         return "Cxx";
+    case Language::None:
+        return "None";
     }
 
     return "";
@@ -228,6 +224,8 @@ const char * toText(Utils::LanguageVersion languageVersion)
         return "CXX2a";
     case LanguageVersion::CXX98:
         return "CXX98";
+    case LanguageVersion::None:
+        return "None";
     }
 
     return "";
@@ -354,9 +352,8 @@ std::ostream &operator<<(std::ostream &out, const SourceLocationEntry &entry)
 
 std::ostream &operator<<(std::ostream &out, const WatcherEntry &entry)
 {
-    out << "("
-        << entry.id << ", "
-        << entry.pathId
+    out << "(" << entry.directoryPathId << ", " << entry.filePathId << ", " << entry.id << ", "
+        << entry.lastModified << ", "
         << ")";
 
     return out;
@@ -398,7 +395,6 @@ std::ostream &operator<<(std::ostream &os, const RequestCompletionsMessage &mess
        << message.filePath << ", "
        << message.line << ", "
        << message.column << ", "
-       << message.projectPartId << ", "
        << message.ticketNumber << ", "
        << message.funcNameStartLine << ", "
        << message.funcNameStartColumn
@@ -714,18 +710,14 @@ std::ostream &operator<<(std::ostream &out, const NativeFilePath &filePath)
 
 std::ostream &operator<<(std::ostream &out, const PrecompiledHeadersUpdatedMessage &message)
 {
-    out << "("
-        << message.projectPartPchs
-        << ")";
+    out << "(" << message.projectPartIds << ")";
 
     return out;
 }
 
 std::ostream &operator<<(std::ostream &out, const ProjectPartPch &projectPartPch)
 {
-    out << "("
-        << projectPartPch.projectPartId << ", "
-        << projectPartPch.pchPath << ", "
+    out << "(" << projectPartPch.projectPartId << ", " << projectPartPch.pchPath << ", "
         << projectPartPch.lastModified << ")";
 
     return out;
@@ -810,19 +802,6 @@ std::ostream &operator<<(std::ostream &out, const ToolTipInfo &info)
     return out;
 }
 
-std::ostream &operator<<(std::ostream &os, const RequestSourceLocationsForRenamingMessage &message)
-{
-    os << "("
-       << message.filePath << ", "
-       << message.line << ", "
-       << message.column << ", "
-       << message.unsavedContent << ", "
-       << message.commandLine
-       << ")";
-
-    return os;
-}
-
 std::ostream &operator<<(std::ostream &os, const RequestSourceRangesAndDiagnosticsForQueryMessage &message)
 {
     os << "("
@@ -853,17 +832,6 @@ std::ostream &operator<<(std::ostream &os, const SourceLocationContainer &contai
     return os;
 }
 
-std::ostream &operator<<(std::ostream &os, const SourceLocationsForRenamingMessage &message)
-{
-    os << "("
-        << message.symbolName << ", "
-        << message.textDocumentRevision << ", "
-        << message.sourceLocations
-        << ")";
-
-    return os;
-}
-
 std::ostream &operator<<(std::ostream &os, const SourceRangeContainer &container)
 {
     os << "("
@@ -949,35 +917,6 @@ std::ostream &operator<<(std::ostream &os, const DocumentVisibilityChangedMessag
     return os;
 }
 
-std::ostream &operator<<(std::ostream &os, const TokenInfo &tokenInfo)
-{
-    os << "(type: " << tokenInfo.types() << ", "
-       << " line: " << tokenInfo.line() << ", "
-       << " column: " << tokenInfo.column() << ", "
-       << " length: " << tokenInfo.length()
-       << ")";
-
-    return  os;
-}
-
-template<class T>
-std::ostream &operator<<(std::ostream &out, const TokenProcessor<T> &tokenInfos)
-{
-    out << "[";
-
-    for (const T &entry : tokenInfos)
-        out << entry;
-
-    out << "]";
-
-    return out;
-}
-
-template
-std::ostream &operator<<(std::ostream &out, const TokenProcessor<TokenInfo> &tokenInfos);
-template
-std::ostream &operator<<(std::ostream &out, const TokenProcessor<FullTokenInfo> &tokenInfos);
-
 std::ostream &operator<<(std::ostream &out, const FilePath &filePath)
 {
     return out << "(" << filePath.path() << ", " << filePath.slashIndex() << ")";
@@ -1103,30 +1042,6 @@ std::ostream &operator<<(std::ostream &out, const RemoveGeneratedFilesMessage &m
     return out << "(" << message.generatedFiles << ")";
 }
 
-std::ostream &operator<<(std::ostream &out, const SuspendResumeJobsEntry &entry)
-{
-    return out << "("
-               << entry.document.filePath() << ", "
-               << entry.jobRequestType << ", "
-               << entry.preferredTranslationUnit
-               << ")";
-}
-
-std::ostream &operator<<(std::ostream &os, const ReferencesResult &value)
-{
-    os << "ReferencesResult(";
-    os << value.isLocalVariable << ", {";
-    for (const SourceRangeContainer &r : value.references) {
-        os << r.start.line << ",";
-        os << r.start.column << ",";
-        EXPECT_THAT(r.start.line, testing::Eq(r.end.line));
-        os << r.end.column - r.start.column << ",";
-    }
-    os << "})";
-
-    return os;
-}
-
 std::ostream &operator<<(std::ostream &out, const SymbolIndexerTask &task)
 {
     return out << "(" << task.filePathId << ", " << task.projectPartId << ")";
@@ -1226,6 +1141,16 @@ std::ostream &operator<<(std::ostream &out, const SourceEntry &entry)
                << typeToString(entry.hasMissingIncludes) << ")";
 }
 
+std::ostream &operator<<(std::ostream &out, const SourceTimeStamp &sourceTimeStamp)
+{
+    return out << "(" << sourceTimeStamp.sourceId << ", " << sourceTimeStamp.timeStamp << ")";
+}
+
+std::ostream &operator<<(std::ostream &out, const TimeStamp &timeStamp)
+{
+    return out << timeStamp.value;
+}
+
 const char *typeToString(IncludeSearchPathType type)
 {
     switch (type) {
@@ -1275,6 +1200,21 @@ std::ostream &operator<<(std::ostream &out, const ProjectPartId &projectPathId)
     return out << projectPathId.projectPathId;
 }
 
+std::ostream &operator<<(std::ostream &out, const PchPaths &pchPaths)
+{
+    return out << "(" << pchPaths.projectPchPath << ", " << pchPaths.systemPchPath << ")";
+}
+
+std::ostream &operator<<(std::ostream &out, const ProjectChunkId &chunk)
+{
+    return out << "(" << chunk.id << ", " << typeToString(chunk.sourceType) << ")";
+}
+
+std::ostream &operator<<(std::ostream &out, const DirectoryPathId &id)
+{
+    return out << id.directoryPathId;
+}
+
 void PrintTo(const FilePath &filePath, ::std::ostream *os)
 {
     *os << filePath;
